package com.mdtexfx.processors;

import org.junit.jupiter.api.Test;

import static java.lang.String.format;
import static org.junit.jupiter.api.Assertions.assertEquals;

/**
 * Verifies that links in a processing chain produce expected transforms.
 */
class AbstractProcessorTest {

  /**
   * Concatenates the class name to the end of the string to process.
   */
  private static class ClassNameProcessor extends ExecutorProcessor<String> {
    public ClassNameProcessor() {
    }

    public ClassNameProcessor( final Processor<String> successor ) {
      super( successor );
    }

    @Override
    public String apply( final String input ) {
      return format( "%s %s", input, toString() );
    }

    public String toString() {
      return getClass().getSimpleName();
    }
  }

  private static class VariableProcessor extends ClassNameProcessor {
    public VariableProcessor( final Processor<String> successor ) {
      super( successor );
    }
  }

  private static class RenderingProcessor extends ClassNameProcessor {
  }

  /**
   * Test that all links in a multiple-link processor chain are invoked.
   */
  @Test
  void test_Chain_MultipleLinks_AllLinksApplied() {
    final var renderProcessor = new RenderingProcessor();
    final var varProcessor = new VariableProcessor( renderProcessor );
    final var execProcessor = new ExecutorProcessor<>( varProcessor );
    final var classname = getClass().getSimpleName();

    final var expected =
        format( "%s %s %s", classname, varProcessor, renderProcessor );
    final var actual = execProcessor.apply( classname );

    assertEquals( expected, actual );
  }
}
